/*
  uf2 flash Basic IMU test. Based on the Basic IMU in the Examples
  Will test if IMU is preasent and detects movement.
  sennds Axcellerometer readings out to Serial port
  the Sensativity can be adjusted via "accelerationThreshold " value.
  1.25 is defualt "nAt" super sensative. Beeps the buzzer when it's over the limit.
  Press the User Button on the Expansion board to lower the Sensativity to movement.
  prints the new value 3.25 way less sensative.

  - Xiao Nrf52840 Sense board.
  - Xiao expansion baord.
  -Add Movement Accel code.

  Prepared by PjGlasso
  built with BSP 2.9.2
  3/25/24
*/


#include <LSM6DS3.h>
#include <Wire.h>
#include <U8x8lib.h>

//Create a instance of class LSM6DS3
LSM6DS3 myIMU(I2C_MODE, 0x6A);    //I2C device address 0x6A  // IMU 
U8X8_SSD1306_128X64_NONAME_HW_I2C u8x8(/* clock=*/ PIN_WIRE_SCL, /* data=*/ PIN_WIRE_SDA, /* reset=*/ U8X8_PIN_NONE);
 // OLEDs without Reset of the Display
float aX, aY, aZ, gX, gY, gZ;
const float accelerationThreshold = 1.25; // threshold of significant in G's
float nAt = accelerationThreshold;
const int numSamples = 59;
int samplesRead = numSamples;
const int buttonPin = 1;     // the number of the pushbutton pin
int buttonState = 0;         // variable for reading the pushbutton status
int BuzzerPin = A3;           //A3 , P0.29 PIN 4
uint8_t interruptCount = 0; // Amount of received interrupts
uint8_t prevInterruptCount = 0; // Interrupt Counter from last loop
int buzzer = BuzzerPin;
int Onthemove = 0; // Not Moving Status
void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
    delay(2000); //relax...
    Serial.println();
    Serial.println("Xiao Expansion board and MCU check & Test compiled on " __DATE__ " at " __TIME__);
	  Serial.println("Processor came out of reset.\n");
    u8x8.begin();
    u8x8.setFlipMode(0);   // set number from 1 to 3, the screen word will rotary 180
    
    pinMode(LED_BUILTIN, OUTPUT);// initialize the LED pin as an output:
    pinMode(buttonPin, INPUT_PULLUP);// initialize the pushbutton pin as an input:
    pinMode(BuzzerPin, OUTPUT);
    pinMode(LEDR, OUTPUT);
    pinMode(LEDG, OUTPUT);
    pinMode(LEDB, OUTPUT);
    setLedRGB(false, false, true); // set blue led


  //while (!Serial);
  //Call .begin() to configure the IMUs
  if (myIMU.begin() != 0) {
    Serial.println("Device error");
     setLedRGB(false, true, false);  // Red
    } else {
    Serial.println("IMU GOOD");
   // getOrientation();
    Serial.print("Temperature: ");
    Serial.print((String) + myIMU.readTempF() + "\u00b0");
    Serial.println();
 

  }
  u8x8.setFont(u8x8_font_8x13B_1x2_r);
    u8x8.clearDisplay();
    u8x8.setCursor(4, 0);
    u8x8.print("Grab Me ");
    u8x8.setCursor(5, 3);
    u8x8.print("&");
    u8x8.setCursor(1, 6);
    u8x8.print("Activity-");
    u8x8.print("Demo");
}

void loop() {
   static uint8_t lastPinState = 1;
    uint8_t pinState = digitalRead(1);
    delay(250);
    if(!pinState && lastPinState){
        onButton();
    }
    lastPinState = pinState;
    
    //while(Serial.available()) Serial.write(Serial.read());
  // wait for significant motion
  while (samplesRead == numSamples) {
    // read the acceleration data
    aX = myIMU.readFloatAccelX();
    aY = myIMU.readFloatAccelY();
    aZ = myIMU.readFloatAccelZ();

    // sum up the absolutes
    float aSum = fabs(aX) + fabs(aY) + fabs(aZ);

    // check if it's above the threshold
    if (aSum >= nAt) {    // checks threshold value.
      // reset the sample read count
      samplesRead = 0;
      setLedRGB(true,false, false); // set RED only
      Onthemove = 1;
      tone (buzzer, 800);
      delay (800);
      noTone(buzzer);
      delay (200);
      u8x8.clearDisplay();
    u8x8.setCursor(5, 3);
    u8x8.print("ON The Move");
      
    break;
    }
  setLedRGB(false, true, false); // set green only
  noTone(buzzer);
  }


   
  // check if the all the required samples have been read since
  // the last time the significant motion was detected
  while (samplesRead < numSamples) {
    // check if both new acceleration and gyroscope data is
    // available
    // read the acceleration and gyroscope data

    samplesRead++;

    // print the data in CSV format
    Serial.print(myIMU.readFloatAccelX(), 3);
    Serial.print(',');
    Serial.print(myIMU.readFloatAccelY(), 3);
    Serial.print(',');
    Serial.print(myIMU.readFloatAccelZ(), 3);
    Serial.print(',');
    Serial.print(myIMU.readFloatGyroX(), 3);
    Serial.print(',');
    Serial.print(myIMU.readFloatGyroY(), 3);
    Serial.print(',');
    Serial.print(myIMU.readFloatGyroZ(), 3);
    Serial.println();

    if (samplesRead == numSamples) {
      // add an empty line if it's the last sample
      Serial.println();
      if (Onthemove != 0 ){
    u8x8.clearDisplay();
    u8x8.setCursor(4, 0);
    u8x8.print("Grab Me ");
    u8x8.setCursor(5, 3);
    u8x8.print("AGAIN");
      }
    }
  }
}
// -------------------- Utilities ------------------------- //

void onButton(){
    String out = "Button pushed ";
    nAt = 3.25; // increase threshold of significant in G's needed to alarm
    out += String(accelerationThreshold);
    Serial.println(out);
    Serial.println(nAt);
    //ble.begin(out);
}

void setLedRGB(bool red, bool green, bool blue) {
  if (!blue) { digitalWrite(LEDB, HIGH); } else { digitalWrite(LEDB, LOW); }
  if (!green) { digitalWrite(LEDG, HIGH); } else { digitalWrite(LEDG, LOW); }
  if (!red) { digitalWrite(LEDR, HIGH); } else { digitalWrite(LEDR, LOW); }
}